
#include "TROOT.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <string>
#include <cmath>
#include <deque>
#include <fstream>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "FFunction.hh"
#include "CMapFunction1D.hh"
#include "CFold.hh"
#include "CAnalysis.hh"
#include "CXmlManager.hh"

#include "CAnalyzeNeut.hh"

CAnalyzeNeut::CAnalyzeNeut(std::string filename)
  : proton(CParticleDefinition::GetParticle("proton")),
    neutron(CParticleDefinition::GetParticle("neutron")),
    chargedparticle(CParticleDefinition::GetParticle("9Li"))
    //      chargedparticle(CParticleDefinition::GetParticle("10Be"))
{
  canvas = new TCanvas("canvas","",200,10,900,900);

  topdir = gDirectory;

  filesim = TFile::Open(filename.c_str(), "READ");
  if(!filesim) abort();

  fileexp = TFile::Open("root/exp.root","recreate");
  if(!fileexp) abort();
}

CAnalyzeNeut::~CAnalyzeNeut()
{
  fileexp->Write();
  fileexp->Close();
  filesim->Close();
}

void CAnalyzeNeut::Main(std::string facility, std::string detector,
			    std::string fragment, std::string target,
			    std::string beam, std::string method, Long64_t nentries)
{
  if(!ReadParamaters(facility, detector, fragment, target)){
    std::cerr << "error occur during reading paramaters" << std::endl;
    return;
  }

  if(!CreateHist()){
    std::cerr << "error occur during creating hist" << std::endl;
    return;
  }    

  if(!CreateExpData(method, nentries)){
    std::cerr << "error occur during creating experiment data" << std::endl;
    return;
  }    

  if(!AnalysisExpData(nentries)){
    std::cerr << "error occur during analyzing experiment data" << std::endl;
    return;
  }    

  if(!DrawHist(beam)){
    std::cerr << "error occur during drawing histogram" << std::endl;
    return;
  }    
}

int CAnalyzeNeut::ReadParamaters(std::string facility, std::string detector,
			     std::string fragment, std::string target)
{
  CXmlManager xml;
  if (!xml.OpenFile("macros/analysis/Analyze/Analyze.xml")) return 0;

  if (!xml.GetValue("//"+facility+"//dpxoverp", &dpxoverp)) return 0;
  if (!xml.GetValue("//"+facility+"//dpyoverp", &dpyoverp)) return 0;
  if (!xml.GetValue("//"+facility+"//dpzoverp", &dpzoverp)) return 0;

  if (!xml.GetValue("//"+fragment+"/"+target+"/dele", &dele)) return 0;
  if (!xml.GetValue("//"+fragment+"/"+target+"/estrag", &estrag)) return 0;
  if (!xml.GetValue("//"+fragment+"/"+target+"/scatter", &multi_theta)) return 0;

  if (!xml.GetValue("//"+detector+"//thrmethod", &thrmethod)) return 0;
  if (!xml.GetValue("//"+detector+"//thr", &thr)) return 0;
  if (!xml.GetValue("//"+detector+"//treso", &treso)) return 0;
  if (!xml.GetValue("//"+detector+"//attlength", &attlength)) return 0;
  if (!xml.GetValue("//"+detector+"//vl", &vl)) return 0;
  if (!xml.GetValue("//"+detector+"//rotation", &rotation)) return 0;
  if (!xml.GetValue("//"+detector+"//length", &length)) return 0;
  if (!xml.GetValue("//"+detector+"//vlength", &vlength)) return 0;
  if (!xml.GetValue("//"+detector+"//width", &width)) return 0;
  if (!xml.GetValue("//"+detector+"//depth", &depth)) return 0;
  if (!xml.GetValue("//"+detector+"//gap", &gap)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth", &zdepth)) return 0;
  if (!xml.GetValue("//"+detector+"//zdepth_back", &zdepth_back)) return 0;
  if (!xml.GetValue("//"+detector+"//shift", &shift)) return 0;
  if (!xml.GetValue("//"+detector+"//multihit", &multihit)) return 0;
  if (!xml.GetValue("//"+detector+"//aminmin", &aminmin)) return 0;
  if (!xml.GetValue("//"+detector+"//ein", &ein)) return 0;
  if (!xml.GetValue("//"+detector+"//sp", &sp)) return 0;
  if (!xml.GetValue("//"+detector+"//step", &step)) return 0;

  return 1;
}

int CAnalyzeNeut::CreateHist()
{
  topdir->cd();

  hamin = new TH1D("rms", "title", 100, 0., 20.);
  htheta_exp = new TH1D("theta_exp", "title", 100, 0., 3.14159);
  htheta_cm_exp = new TH1D("theta_cm_exp", "title", 100, 0., 3.14159);

  htheta_lab = new TH1D("theta_lab", "title", 100, -2., 2);
  hphi_lab = new TH1D("phi_lab", "title", 100, -2., 2);

  hthetaenergy_lab
    = new TH2D("thetaenergy_lab", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_lab_gate
    = new TH2D("thetaenergy_lab_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_cm
    = new TH2D("thetaenergy_cm", "title", 100, 0., 3.14159, 100, 0., ein+20);
  hthetaenergy_cm_gate
    = new TH2D("thetaenergy_cm_gate", "title", 100, 0., 3.14159, 100, 0., ein+20);

  hdee_lab
    = new TH2D("dee_lab", "title", 100, 0., 100, 100, 0., ein+20);
  hthetade_lab
    = new TH2D("thetade_lab", "title", 100, 0., 3.14159, 100, 0., 100);
  hthetadee_lab
    = new TH3D("thetadee_lab", "title",
	       100, 0., 3.14159,
	       100, 0., 100,
	       100, 0., ein+20);

  htheta_cm = new TH1D("theta_cm", "title", 100, 0., 3.14159);
  htheta_cm_gate = new TH1D("theta_cm_gate", "title", 100, 0., 3.14159);

  hpx = new TH1D("n_px", "title", 100, 0.-2, 0.+2);
  hpy = new TH1D("n_py", "title", 100, 0.-2, 0.+2);
  hpz = new TH1D("n_pz", "title", 100, 0.-2, 0.+2);
  hcpx = new TH1D("cp_px", "title", 100, 0.-2, 0.+2);
  hcpy = new TH1D("cp_py", "title", 100, 0.-2, 0.+2);
  hcpz = new TH1D("cp_pz", "title", 100, 0.-2, 0.+2);
  herel = new TH1D("Erel", "title", 50, 0., 1.);

  return 1;
}

int CAnalyzeNeut::CreateExpData(std::string method, Long64_t nentries)
{
  TTree* tree;
  filesim->GetObject("Koyomi",tree);

  fileexp->cd();
  TTree* newtree = new TTree("Hitagi", "title");

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(tree->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return 0;

  CDataHime* dataexp = new CDataHime;
  newtree->Branch("hime", &dataexp);

  topdir->cd();

  Long64_t count = 0;
  if(nentries < 0) nentries = tree->GetEntriesFast();
  for(int i=0; i<nentries; i++) {
    ++count;
    if(count%10000 == 0) std::cout << "\r converted events:" << count << std::flush;

    Long64_t entry = tree->LoadTree(i);
    bdatasim->GetEntry(entry);

    if(!datasim->fEventFlag) continue;

    dataexp->Reset();
    SimToExp(datasim, dataexp, method);

    newtree->Fill();
  }
  std::cout << "\r converted events:" << count << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int CAnalyzeNeut::AnalysisExpData(Long64_t nentries)
{
  TTree* treesim;
  filesim->GetObject("Koyomi",treesim);

  CDataHimeSim* datasim = 0;
  TBranch *bdatasim = 0;
  if(treesim->SetBranchAddress("hime",&datasim,&bdatasim) == TTree::kMissingBranch) return 0;

  TTree* treeexp;
  fileexp->GetObject("Hitagi",treeexp);

  CDataHime* dataexp = 0;
  TBranch *bdataexp = 0;
  if(treeexp->SetBranchAddress("hime",&dataexp,&bdataexp) == TTree::kMissingBranch) return 0;

  Long64_t count = 0;
  if(nentries < 0) nentries = treesim->GetEntriesFast();
  for(Long64_t i=0, j=0; i<nentries; i++) {
    Long64_t entrysim = treesim->LoadTree(i);
    bdatasim->GetEntry(entrysim);

    if(!datasim->fEventFlag) continue;

    Long64_t entryexp = treeexp->LoadTree(j++);
    bdataexp->GetEntry(entryexp);

    //protonの真の角度分布をグラフ化してそれと比較する
    Double_t theta_lab, phi_lab, energy_lab;
    Double_t theta_cm, energy_cm;
    if(GetFirstProtonInformation(datasim, proton) != 0){
      theta_lab = proton.theta();
      phi_lab = proton.phi();
      energy_lab = proton.k();

      neutron.p(datasim->fMomentum.x(),datasim->fMomentum.y(),datasim->fMomentum.z());
      Double_t beta = neutron.p()/(neutron.e()+proton.m());
      CKinema::LorentzBoost(proton, beta, 2);

      theta_cm = proton.theta();
      energy_cm = proton.k();

      htheta_cm->Fill(theta_cm);
      hthetaenergy_lab->Fill(theta_lab, energy_lab);
      hthetaenergy_cm->Fill(theta_cm, energy_cm);
      if(dataexp->fEventFlag){
	htheta_lab->Fill(theta_lab-dataexp->fPrp.Theta());
	hphi_lab->Fill(phi_lab-dataexp->fPrp.Phi());

	htheta_cm_gate->Fill(theta_cm);
	hthetaenergy_lab_gate->Fill(theta_lab, energy_lab);
	hthetaenergy_cm_gate->Fill(theta_cm, energy_cm);

	hdee_lab->Fill(GetTotalEnergyLoss(dataexp),
		       energy_lab);
	hthetade_lab->Fill(theta_lab,
			   GetTotalEnergyLoss(dataexp));
	hthetadee_lab->Fill(theta_lab,
			    GetTotalEnergyLoss(dataexp),
			    energy_lab);
      }
    }

    if(dataexp->fEventFlag){
      ++count;
      if(count%100 == 0) std::cout << "\r analysed events:" << count << std::flush;
    }else{
      continue;
    }

    TVector3 np = datasim->fMomentum;
    TVector3 cpp = datasim->fMomentumChargedParticle;

    //fold momentum
    neutron.p(dataexp->fP.Px(), dataexp->fP.Py(), dataexp->fP.Pz());
    FoldChargedParticle(cpp, chargedparticle,
			TVector3(dpxoverp, dpyoverp, dpzoverp),
			dele, estrag, multi_theta);

    double erel = CKinema::InvariantMass(neutron, chargedparticle)
      - neutron.m() - chargedparticle.m();
    herel->Fill(erel);

    hpx->Fill(100*(neutron.p(0)-np.Px())/np.Mag());
    hpy->Fill(100*(neutron.p(1)-np.Py())/np.Mag());
    hpz->Fill(100*(neutron.p(2)-np.Pz())/np.Mag());

    hcpx->Fill(100*(chargedparticle.p(0)-cpp.Px())/cpp.Mag());
    hcpy->Fill(100*(chargedparticle.p(1)-cpp.Py())/cpp.Mag());
    hcpz->Fill(100*(chargedparticle.p(2)-cpp.Pz())/cpp.Mag());
  }
  std::cout << "\r analysed events:" << count << std::endl;

  treeexp->ResetBranchAddresses();
  treesim->ResetBranchAddresses();
    
  topdir->cd();

  return 1;
}

int CAnalyzeNeut::DrawHist(std::string beam)
{
  canvas->Divide(3,3);
  //    canvas->Divide(4,3);
  canvas->cd(1);
  hpx->Draw();
  hpx->Fit("gaus");
  canvas->cd(2);
  hpy->Draw();
  hpy->Fit("gaus");
  canvas->cd(3);
  hpz->Draw();
  hpz->Fit("gaus");
  canvas->cd(4);
  hcpx->Draw();
  hcpx->Fit("gaus");
  canvas->cd(5);
  hcpy->Draw();
  hcpy->Fit("gaus");
  canvas->cd(6);
  hcpz->Draw();
  hcpz->Fit("gaus");

  canvas->cd(7);
  herel->Draw();

  if(beam == "11Li"){
    CMapFunction1D* mapf = new CMapFunction1D;
    mapf->SetFile("txt/map.txt");
    TF1* f2m1p2p = new TF1("f2m1p2p",
			   mapf,
			   static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			   (&CMapFunction1D::VRBW2MF1D),
			   0.01, 1.0, 12);
    CFold* fold = new CFold(f2m1p2p);
    TF1* ff2m1p2p = fold->GetFoldedFunction();
    ff2m1p2p->SetParameter(0, 3.5e8);
    ff2m1p2p->SetParameter(1, -12.7);
    ff2m1p2p->FixParameter(2, 3.0);
    ff2m1p2p->FixParameter(3, 0.325/2.);
    ff2m1p2p->FixParameter(4, neutron.m());
    ff2m1p2p->FixParameter(5, chargedparticle.m());
    ff2m1p2p->SetParameter(6, 1.96e-6);
    ff2m1p2p->SetParameter(7, 0.22);
    ff2m1p2p->SetParameter(8, 0.09);
    ff2m1p2p->SetParLimits(8, 0.05,0.2);
    ff2m1p2p->SetParameter(9, 7.27e-6);
    ff2m1p2p->SetParameter(10, 0.53);
    ff2m1p2p->SetParameter(11, 0.45);
    ff2m1p2p->SetParLimits(11, 0.3,6);
    ff2m1p2p->Draw("same");
    TFitResultPtr frpf2m1p2p = herel->Fit(ff2m1p2p, "Sb", "", 0.01, 0.8);

    TF1* f2m = new TF1("f2m",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::VRMF1D),
		       0.01, 1.0, 6);
    CFold* foldf2m = new CFold(f2m);
    TF1* ff2m = foldf2m->GetFoldedFunction();
    ff2m->SetParameter(0, frpf2m1p2p->Parameter(0));
    ff2m->SetParameter(1, frpf2m1p2p->Parameter(1));
    ff2m->SetParameter(2, frpf2m1p2p->Parameter(2));
    ff2m->SetParameter(3, frpf2m1p2p->Parameter(3));
    ff2m->SetParameter(4, frpf2m1p2p->Parameter(4));
    ff2m->SetParameter(5, frpf2m1p2p->Parameter(5));
    ff2m->SetLineStyle(2);
    ff2m->DrawCopy("same");

    TF1* f1p = new TF1("f1p",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::BWMF1D),
		       0.01, 1.0, 6);
    CFold* foldf1p = new CFold(f1p);
    TF1* ff1p = foldf1p->GetFoldedFunction();
    ff1p->SetParameter(0, frpf2m1p2p->Parameter(0)*frpf2m1p2p->Parameter(6));
    ff1p->SetParameter(1, frpf2m1p2p->Parameter(7));
    ff1p->SetParameter(2, frpf2m1p2p->Parameter(8));
    ff1p->SetLineStyle(3);
    ff1p->DrawCopy("same");

    TF1* f2p = new TF1("f2p",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::BWMF1D),
		       0.01, 1.0, 6);
    CFold* foldf2p = new CFold(f2p);
    TF1* ff2p = foldf2p->GetFoldedFunction();
    ff2p->SetParameter(0, frpf2m1p2p->Parameter(0)*frpf2m1p2p->Parameter(9));
    ff2p->SetParameter(1, frpf2m1p2p->Parameter(10));
    ff2p->SetParameter(2, frpf2m1p2p->Parameter(11));
    ff2p->SetLineStyle(4);
    ff2p->DrawCopy("same");
  }else if(beam == "11Be"){
    CMapFunction1D* mapf = new CMapFunction1D;
    mapf->SetFile("txt/map.txt");
    TF1* f2m = new TF1("f2m",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::VRMF1D),
		       0.01, 1.0, 6);
    CFold* fold = new CFold(f2m);
    TF1* ff2m = fold->GetFoldedFunction();
    ff2m->SetParameter(0, 9e7);
    ff2m->SetParameter(1, -12.7);
    ff2m->FixParameter(2, 3.0);
    ff2m->FixParameter(3, 0.325/2.);
    ff2m->FixParameter(4, neutron.m());
    ff2m->FixParameter(5, chargedparticle.m());
    TFitResultPtr frpf2m = herel->Fit(ff2m, "b", "", 0.01, 0.5);
  }else if(beam == "12B"){
    CMapFunction1D* mapf = new CMapFunction1D;
    mapf->SetFile("txt/map.txt");
    TF1* f1p2p = new TF1("fp",
			 mapf,
			 static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
			 (&CMapFunction1D::BW2MF1D),
			 0.05, 0.8, 6);
    CFold* fold = new CFold(f1p2p);
    TF1* ff1p2p = fold->GetFoldedFunction();
    ff1p2p->SetParameter(0, 200);
    ff1p2p->SetParameter(1, 10);
    ff1p2p->SetParameter(2, 0.22);
    ff1p2p->SetParameter(3, 0.09);
    ff1p2p->SetParameter(4, 0.53);
    ff1p2p->SetParameter(5, 0.45);
    TFitResultPtr frpf1p2p = herel->Fit(ff1p2p, "S", "", 0.05, 0.8);

    TF1* f1p = new TF1("f1p",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::BWMF1D),
		       0.01, 1.0, 6);
    CFold* foldf1p = new CFold(f1p);
    TF1* ff1p = foldf1p->GetFoldedFunction();
    ff1p->SetParameter(0, frpf1p2p->Parameter(0)*frpf1p2p->Parameter(1));
    ff1p->SetParameter(1, frpf1p2p->Parameter(2));
    ff1p->SetParameter(2, frpf1p2p->Parameter(3));
    ff1p->SetLineColor(4);
    ff1p->Draw("same");

    TF1* f2p = new TF1("f2p",
		       mapf,
		       static_cast<Double_t (CMapFunction1D::*)(Double_t*, Double_t*)>
		       (&CMapFunction1D::BWMF1D),
		       0.01, 1.0, 6);
    CFold* foldf2p = new CFold(f2p);
    TF1* ff2p = foldf2p->GetFoldedFunction();
    ff2p->SetParameter(0, frpf1p2p->Parameter(0));
    ff2p->SetParameter(1, frpf1p2p->Parameter(4));
    ff2p->SetParameter(2, frpf1p2p->Parameter(5));
    ff2p->SetLineColor(5);
    ff2p->Draw("same");
  }else{
    std::cout << "\nbeam:" << beam << " is not yet defined\n" << std::endl;
  }

  canvas->cd(8);
  htheta_lab->Draw();
  htheta_lab->Fit("gaus", "", "same");

  canvas->cd(9);
  hphi_lab->Draw();
  hphi_lab->Fit("gaus", "", "same");

  canvas->Update();

  return 1;
}

void CAnalyzeNeut::SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method)
{
  std::deque<CDataHimeSim::OneData>::iterator itsim;
  itsim = simdata->fDequeOfOneData.begin();
  while(itsim != simdata->fDequeOfOneData.end()){
    std::vector<Int_t> id;
    id.push_back(itsim->fIdMother);
    id.push_back(itsim->fId);

    Double_t xl = length/2.+itsim->fPosition.X()-shift;
    Double_t xr = length/2.-itsim->fPosition.X()+shift;
    Double_t yl = length/2.+itsim->fPosition.Y();
    Double_t yr = length/2.-itsim->fPosition.Y();

    Double_t fEdepee = CKinema::MeVtoMeVee(itsim->fKineticEnergy,
					   itsim->fKineticEnergy - itsim->fEdep,
					   itsim->fParticleName.Data());

    Double_t tl, tr, al, ar;
    if(rotation == 1){
      if(id[0]%2){
	tl = itsim->fTime + yl/vl;
	tr = itsim->fTime + yr/vl;
	al = fEdepee*exp(-yl/attlength);
	ar = fEdepee*exp(-yr/attlength);
      }else{
	tl = itsim->fTime + xl/vl;
	tr = itsim->fTime + xr/vl;
	al = fEdepee*exp(-xl/attlength);
	ar = fEdepee*exp(-xr/attlength);
      }
    }else{
      tl = itsim->fTime + xl/vl;
      tr = itsim->fTime + xr/vl;
      al = fEdepee*exp(-xl/attlength);
      ar = fEdepee*exp(-xr/attlength);
    }      

    CDataHime::OneData* onedata = dataexp->Find(id[0], id[1]);
    if(onedata){
      if(onedata->fTL > tl) onedata->fTL = tl;
      if(onedata->fTR > tr) onedata->fTR = tr;
      onedata->fAL += al;
      onedata->fAR += ar;
    }else{
      dataexp->fDequeOfOneData.push_back(CDataHime::OneData(id, tl, tr, al, ar));
    }

    ++itsim;
  }

  std::deque<CDataHime::OneData>::iterator it;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){

    //thr判定その1
    if(thrmethod == 1){
      if(it->fAL < thr || it->fAR < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    it->fTL += gRandom->Gaus(0,treso);
    it->fTR += gRandom->Gaus(0,treso);  
    
    it->fT = (it->fTL + it->fTR - length/vl)/2;
    it->fE = sqrt(it->fAL*it->fAR)*exp(length/(2.*attlength));
    
    //thr判定その2
    if(thrmethod == 2){
      if(it->fE < thr){
	it = dataexp->fDequeOfOneData.erase(it);
	continue;
      }
    }

    Double_t x, y, z;
    if(rotation == 1){
      if(it->fId[0]%2){
	x = it->fId[1]*width + width/2. - vlength/2. + shift;
	y = vl*(it->fTL-it->fTR)/2.;
      }else{
	x = vl*(it->fTL-it->fTR)/2. + shift;
	y = it->fId[1]*width + width/2. - vlength/2.;
      }
    }else{
      x = vl*(it->fTL-it->fTR)/2. + shift;
      y = it->fId[1]*width + width/2. - vlength/2.;
    }
    z = it->fId[0]*(depth+gap) + zdepth + depth/2.;

    it->fX.SetXYZ(x, y, z);

    ++it;
  }

  if(method == "Track"){
    Track(dataexp);
  }else if(method == "Simple"){
    Simple(dataexp);
  }else if(method == "Niren"){
    Niren(dataexp);
  }else{
    std::cerr << "method:" << method << " cannot use" << std::endl;
    return;
  }
}

void CAnalyzeNeut::Track(CDataHime* dataexp)
{
#ifdef VISUALIZE
  TGraph2DErrors* graph = new TGraph2DErrors;//staticを付けるとメモリの再確保がなくなり早い
#endif

  if(dataexp->fDequeOfOneData.empty()) return;

  //どうせ何回も使うので
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  //最速イベントではなく先頭イベント。
  //最速を選ぶと時間分解能の分だけ微妙にシフトすると思われる。
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    ++it;
  }

  //遅いイベントは関係ないので切り捨て。
  it = dataexp->fDequeOfOneData.begin();
  double tmin = itmin->fT;//roop内部でeraseを使ってるので途中でitminが不正になりうる。
  while(it != dataexp->fDequeOfOneData.end()){
    //先頭でhitしてからある程度時間がかかったeventは切ってしまう
    //時間で排除する代わりに、シグナルの近傍度みたいな量を定義して、
    //それでcutしたほうがノイズを効率的に落とせるかも
    if(abs(it->fT - tmin) > 1.2){
      it = dataexp->fDequeOfOneData.erase(it);
      continue;
    }

    ++it;
  }

  //eraseしたので最速イベントのイテレータを取り直す。(必須!!)
  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(it->fX.Z() < itmin->fX.Z()) itmin = it;
    ++it;
  }

  //hit数に満たない場合はcut
  if((int)dataexp->fDequeOfOneData.size() < multihit){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //2Dfit(zの誤差を無視する)でtracking
  Int_t point = 0;
  std::vector<double> pointx, pointy, pointz;
  std::vector<double> pointxe, pointye, pointze;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    pointx.push_back(it->fX.X());
    pointy.push_back(it->fX.Y());
    pointz.push_back(it->fX.Z());
    if(it->fId[0]%2){
      pointxe.push_back(width/sqrt(12));
      pointye.push_back(vl*treso/sqrt(2));
    }else{	 
      pointxe.push_back(vl*treso/sqrt(2));
      pointye.push_back(width/sqrt(12));
    }
    pointze.push_back(depth/sqrt(12));

#ifdef VISUALIZE
    graph->SetPoint(point, it->fX.X(), it->fX.Y(), it->fX.Z());
    if(it->fId[0]%2){
      graph->SetPointError(point, width/sqrt(12), vl*treso/sqrt(2), depth/sqrt(12));
    }else{	 
      graph->SetPointError(point, vl*treso/sqrt(2), width/sqrt(12), depth/sqrt(12));
    }
#endif

    ++point;
    ++it;
  }
  double parFit[4];

  CAnalysis::LinearFit(pointz, pointx, pointxe, parFit[0], parFit[1]);
  CAnalysis::LinearFit(pointz, pointy, pointye, parFit[2], parFit[3]);
  double aminx = CAnalysis::SD(pointz, pointx, parFit[0], parFit[1]);
  double aminy = CAnalysis::SD(pointz, pointy, parFit[2], parFit[3]);
  double amin = sqrt(aminx*aminx+aminy*aminy);//rmsっぽいもの

  //rmsを収集
  hamin->Fill(amin);

  //理想的には選択的にトラックを引くアルゴリズムが必要。
  //たとえばamin<5になるまで、計算に入れる点を変えるとか。
  //今はaminminで単純に切り捨てごめん。
  if(amin > aminmin){
    dataexp->fEventFlag = 0;//in-validなデータの場合は0
    return;
  }

  //analysis of timing
  TVector3 xf(parFit[1], parFit[3], 1.);//fitした直線にそったベクトル
  double theta = xf.Angle(itmin->fX);
  double theta_rp = xf.Theta();
  double phi_rp = xf.Phi();

  // derive recoile proton energy from theta(lab)
  CParticle pin = CParticleDefinition::GetParticle("proton");
  CParticle nin = CParticleDefinition::GetParticle("neutron");
  CParticle pout = CParticleDefinition::GetParticle("proton");
  CParticle nout = CParticleDefinition::GetParticle("neutron");

  nin.k(ein-sp);//一陽子分離エネルギーを考慮。ほとんど効かないけど。
  pin.k(0);//Fermi motionは無視して静止していると考える。

  //lab系のthetaをパラメータとして、剛体の2体散乱をさせる。
  Double_t theta_cm = CKinema::Scatter(theta, 0., nin, pin, pout, nout);

  //pout,noutの角度をlab系にする(thetaはlab系でのneutronとの角度の差)
  pout.thetaphi(theta_rp, phi_rp);
  nout.thetaphibar(theta_rp, phi_rp);

  //recoil protonの情報の保存
  dataexp->fPrp.SetPxPyPzE(pout.p(0), pout.p(1), pout.p(2), pout.e());

  //thetaを取得
  htheta_exp->Fill(theta);
  htheta_cm_exp->Fill(theta_cm);

  //timingの解析の本体
  double timing = 0;
  int count = 0;
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    double tsum = -1.;
    if(it == itmin){
      tsum = 0;
      //先頭でhitしてから4層通過分程度時間がかかったeventは時間の計算には参加させない
    }else if(abs(it->fT - itmin->fT) < 0.8){
      double xtemp, ytemp, ztemp;
      if(it->fId[0]-itmin->fId[0] > 0){
	//二層め以降で成ったシグナルは荷電粒子によるものと思われ、
	//シミュレーション的にはシンチレータの端面で鳴るのでZを補正。
	//実験でも基本的に同じ補正でいいはずだが未知
	CAnalysis::line(it->fX.Z()-depth/2., parFit, xtemp, ytemp, ztemp);
      }else{
	CAnalysis::line(it->fX.Z(), parFit, xtemp, ytemp, ztemp);	
      }
      TVector3 X(xtemp, ytemp, ztemp);
      CAnalysis::line(itmin->fX.Z(), parFit, xtemp, ytemp, ztemp);
      TVector3 Xmin(xtemp, ytemp, ztemp);
      TVector3 distance_v = X - Xmin;
      double distance = distance_v.Mag();

      CParticle ptemp = pout;
      tsum  = CKinema::BetheBloch::TimeOut
	("p", "Scint", ptemp,
	 distance - (it->fId[0]-itmin->fId[0])*gap/cos(distance_v.Theta()), step); //空間の分を引く
      if(tsum >= 0.){
	//ほんとは2cm+2cm+2cm+....と交互に時間を足していかなければならない。
	//ここで少しmeanのずれが生じていると考えられる。
	tsum += (distance/2.)/((pout.beta()+ptemp.beta())*CKinema::fLightVelocity/2.);
      }
    }
    if(tsum >= 0){
      //補正した結果、改善する方向に向かわない場合は補正に参加させない
      if(abs((it->fT - tsum) - itmin->fT) < treso*2){
	timing += it->fT - tsum;
	++count;
      }
    }
    ++it;
  }
  timing /= count;

  //位置はlinearfitで求まった先頭、時間は上の解析で補正した時間
  double xtemp, ytemp, ztemp;
  CAnalysis::line(itmin->fX.Z(),parFit,xtemp,ytemp,ztemp);
  dataexp->fX.SetXYZT(xtemp, ytemp, ztemp, timing);

  //X,Y,Z,Tが求まったらMomentumを計算しておく。
  //この関数の内部で非数が渡されてないかエラーチェックが走る
  dataexp->CalMomentum();

#ifdef VISUALIZE
  graph->SetMarkerStyle(8);
  graph->SetMarkerSize(1);
  graph->GetXaxis()->SetLimits(-length/2., length/2.);
  graph->GetYaxis()->SetLimits(-length/2., length/2.);
  graph->SetMinimum(zdepth);
  graph->SetMaximum(zdepth+zdepth_back);

  if(0){
    graph->Draw("PERR");
  }else{
    static bool init = true;
    if(init){
      graph->Draw("PERR");
      init = false;
    }else{
      graph->Draw("PERRSAME");
    }
  }

  CAnalysis::DrawLine(parFit, kRed, zdepth, zdepth+zdepth_back);

  TCanvas* canvas = (TCanvas*)gROOT->GetListOfCanvases()->FindObject("canvas");
  canvas->Update();

  CAnalysis::EventLoop();
#endif

  dataexp->fEventFlag = 1;//validなデータの場合は1

  return;
}

void CAnalyzeNeut::Simple(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  dataexp->fX.SetXYZT(itmin->fX.X(), itmin->fX.Y(), itmin->fX.Z(), itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}

void CAnalyzeNeut::Niren(CDataHime* dataexp)
{
  static std::deque<CDataHime::OneData>::iterator it;
  static std::deque<CDataHime::OneData>::iterator itmin;

  if(dataexp->fDequeOfOneData.empty()){
    dataexp->fEventFlag = 0;
    return;
  }

  itmin = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    if(itmin->fX.Z() > it->fX.Z()) itmin = it;
    ++it;
  }

  //ここのアルゴリズムがかなり適当なのでだめだめ。
  //本気で2連hitを使うならdtから得たyとの整合性を取ったりしないとだめすぎる。
  Double_t zmin = itmin->fX.Z() - zdepth+1;
  Int_t idmin = zmin/2./depth+0.1;
  std::deque<CDataHime::OneData>::iterator itmin2;
  itmin2 = dataexp->fDequeOfOneData.begin();
  it = dataexp->fDequeOfOneData.begin();
  while(it != dataexp->fDequeOfOneData.end()){
    Double_t z = it->fX.Z() - (zdepth+1);
    Int_t id = z/2./depth+0.1;
    if(idmin+1 == id){
      itmin2 = it;
      break;
    }
    ++it;
  }
  
  if(itmin2 == dataexp->fDequeOfOneData.begin()){
    dataexp->fEventFlag = 0;
    return;
  }
  
  if(idmin%2){
    dataexp->fX.SetX(itmin->fX.X());
    dataexp->fX.SetY(itmin2->fX.Y());
  }else{
    dataexp->fX.SetX(itmin2->fX.X());
    dataexp->fX.SetY(itmin->fX.Y());
  }
  dataexp->fX.SetZ(itmin->fX.Z());

// ノイズが多すぎてうまく行かない
//  Double_t meanbeta = 0.6;
//  Double_t meanlength = 4.;
//  Double_t time;
//  if(itmin->fX.Z() - itmin2->fX.Z() < 1.){
//    time = (itmin->fT + itmin2->fT)/2.;
//  }else{    
//    time = (itmin->fT + itmin2->fT)/2. - meanlength/(meanbeta*CKinema::fLightVelocity);
//  }
//  dataexp->fX.SetT(time);

  //シンプルに先頭の時間
  dataexp->fX.SetT(itmin->fT);

  dataexp->CalMomentum();

  dataexp->fEventFlag = 1;

  return;
}

// independent functions

int CAnalyzeNeut::GetFirstProtonInformation(CDataHimeSim* data, CParticle& proton)
{
  static std::deque<CDataHimeSim::OneData>::iterator itsim;
  static std::deque<CDataHimeSim::OneData>::iterator itmin;
  itsim = (data->fDequeOfOneData).begin();
  itmin = (data->fDequeOfOneData).end();
  Double_t tmin = 1e+10;
  while(itsim != data->fDequeOfOneData.end()){
    if(itsim->fParticleName == "proton"){
      if(tmin > itsim->fTime){
	tmin = itsim->fTime;
	itmin = itsim;
      }
    }
    ++itsim;
  }

  if(itmin == data->fDequeOfOneData.end()){
    return 0;
  }else{
    proton.p(itmin->fMomentum.x(),
	     itmin->fMomentum.y(),
	     itmin->fMomentum.z());

    return 1;
  }
}

double CAnalyzeNeut::GetTotalEnergyLoss(CDataHime* data)
{
  static std::deque<CDataHime::OneData>::iterator it;
  it = (data->fDequeOfOneData).begin();
  double energy=0.;

  while(it != data->fDequeOfOneData.end()){
    energy += it->fE;
    ++it;
  }

  return energy;
}

void CAnalyzeNeut::FoldChargedParticle(TVector3 momentum, CParticle& particle,
			 TVector3 dpoverp, double dele, double estrag, double multi_theta)
{
  static TRandom3 random;

  particle.p(momentum.Px(),
	     momentum.Py(),
	     momentum.Pz());

  double detodp_coef = (particle.k()+particle.m())/(particle.k()+2*particle.m())
    /particle.k()*particle.p();
  double sigma[] = {momentum.Mag()*dpoverp.x(),
		    momentum.Mag()*dpoverp.y(),
		    momentum.Mag()*dpoverp.z(),
		    dele*detodp_coef,
		    estrag*detodp_coef,
		    multi_theta*momentum.Pz()};

  //energy loss difference
  momentum.SetZ(momentum.Pz()+random.Uniform(-sigma[3]/2., sigma[3]/2.));
  //multiple scattering and energy stragglling
  momentum.SetXYZ(random.Gaus(momentum.Px(), sigma[5]),
		  random.Gaus(momentum.Py(), sigma[5]),
		  random.Gaus(momentum.Pz(), sigma[4]));
  //detector resolution
  particle.p(random.Gaus(momentum.Px(), sigma[0]),
	     random.Gaus(momentum.Py(), sigma[1]),
	     random.Gaus(momentum.Pz(), sigma[2]));
}
