
#include "TH1.h"
#include "TH1D.h"
#include "TH2.h"
#include "TProfile.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <iomanip>
#include <string>

#include "CCanvasManager.hh"
#include "CHistManager.hh"
#include "NGlobal.hh"

#include "CSimpleFunction.hh"

TFitResultPtr CSimpleFunction::Fit(const char* formula, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  //  if(!NAlias::histmanager->Check()) return TFitResultPtr();
  //
  TH1* h = NAlias::histmanager->GetHist();
  if(!NAlias::histmanager->Check(h))   return TFitResultPtr();

  return h->Fit(formula, option, goption, xmin, xmax);
}

TFitResultPtr CSimpleFunction::FitPolynomial(int n, Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  std::ostringstream os;
  os << n;
  std::string pol("pol" + os.str());

  return Fit(pol.c_str(), option, goption, xmin, xmax);
}

TFitResultPtr CSimpleFunction::FitExponential(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("expo", option, goption, xmin, xmax);
}

TFitResultPtr CSimpleFunction::FitGaussian(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("gaus", option, goption, xmin, xmax);
}

TFitResultPtr CSimpleFunction::FitLandau(Option_t* option, Option_t* goption, Double_t xmin, Double_t xmax)
{
  return Fit("landau", option, goption, xmin, xmax);
}

TProfile* CSimpleFunction::ProfileX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;
  
  TH1* hist = NAlias::histmanager->GetHist();

  if(!(hist && hist->InheritsFrom("TH2"))){
    std::ostringstream os;
    os << FunctionName("ProfileX") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_pfx";
  NGlobal::IncrementName(name);

  TProfile* prof = hist2->ProfileX(name.c_str(), firstybin, lastybin, option);

  std::string title = NAlias::histmanager->GetNameX();
  prof->SetTitle(title.c_str());

  prof->SetMinimum(hist2->GetYaxis()->GetXmin());
  prof->SetMaximum(hist2->GetYaxis()->GetXmax());

  NAlias::histmanager->InsertAfterCurrent(prof->GetName(), prof->GetTitle());

  prof->ResetAttFill();
  prof->SetOption("E");

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return prof;
}

TProfile* CSimpleFunction::ProfileY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;
  
  TH1* hist = NAlias::histmanager->GetHist();

  if(hist && hist->InheritsFrom("TH2")){
    std::ostringstream os;
    os << FunctionName("ProfileY") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_pfy";
  NGlobal::IncrementName(name);

  TProfile* prof = hist2->ProfileY(name.c_str(), firstxbin, lastxbin, option);

  std::string title = NAlias::histmanager->GetNameY();
  prof->SetTitle(title.c_str());

  prof->SetMinimum(hist2->GetXaxis()->GetXmin());
  prof->SetMaximum(hist2->GetXaxis()->GetXmax());

  NAlias::histmanager->InsertAfterCurrent(prof->GetName(), prof->GetTitle());

  prof->ResetAttFill();
  prof->SetOption("E");

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return prof;
}

TH1D* CSimpleFunction::ProjectionX(Int_t firstybin, Int_t lastybin, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;
  
  TH1* hist = NAlias::histmanager->GetHist();

  if(!(hist && hist->InheritsFrom("TH2"))){
    std::ostringstream os;
    os << FunctionName("ProjectionX") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_px";
  NGlobal::IncrementName(name);


  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  std::string title = NAlias::histmanager->GetNameX();
  histd->SetTitle(title.c_str());

  NAlias::histmanager->InsertAfterCurrent(histd->GetName(), histd->GetTitle());

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return histd;
}

TH1D* CSimpleFunction::ProjectionY(Int_t firstxbin, Int_t lastxbin, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;
  
  TH1* hist = NAlias::histmanager->GetHist();

  if(!(hist && hist->InheritsFrom("TH2"))){
    std::ostringstream os;
    os << FunctionName("ProjectionY") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_py";
  NGlobal::IncrementName(name);

  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  std::string title = NAlias::histmanager->GetNameY();
  histd->SetTitle(title.c_str());

  NAlias::histmanager->InsertAfterCurrent(histd->GetName(), histd->GetTitle());

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return histd;
}

TH1D* CSimpleFunction::BanY(double firsty, double lasty, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;

  TH1* hist = NAlias::histmanager->GetHist();

  if(!(hist && hist->InheritsFrom("TH2"))){
    std::ostringstream os;
    os << FunctionName("BanY") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_bny";
  NGlobal::IncrementName(name);

  Int_t firstybin = hist2->GetYaxis()->FindBin(firsty);
  Int_t lastybin  = hist2->GetYaxis()->FindBin(lasty);

  TH1D* histd = hist2->ProjectionX(name.c_str(), firstybin, lastybin, option);

  std::string title = NAlias::histmanager->GetNameX();
  histd->SetTitle(title.c_str());

  NAlias::histmanager->InsertAfterCurrent(histd->GetName(), histd->GetTitle());

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return histd;
}

TH1D* CSimpleFunction::BanX(double firstx, double lastx, Option_t* option)
{
  if(!NAlias::histmanager->Check()) return 0;

  TH1* hist = NAlias::histmanager->GetHist();

  if(!(hist && hist->InheritsFrom("TH2"))){
    std::ostringstream os;
    os << FunctionName("BanX") << "not 2d hist\n" << std::endl;
    NGlobal::Message(os.str(), kMID);

    return 0;
  }

  TH2* hist2 = (TH2*)hist;

  std::string name(hist2->GetName());
  name += "_bnx";
  NGlobal::IncrementName(name);

  Int_t firstxbin = hist2->GetXaxis()->FindBin(firstx);
  Int_t lastxbin  = hist2->GetXaxis()->FindBin(lastx);

  TH1D* histd = hist2->ProjectionY(name.c_str(), firstxbin, lastxbin, option);

  std::string title = NAlias::histmanager->GetNameY();
  histd->SetTitle(title.c_str());

  NAlias::histmanager->InsertAfterCurrent(histd->GetName(), histd->GetTitle());

  NAlias::canvasmanager->NextSubpad();
  NAlias::histmanager->DrawCurrent();

  return histd;
}

std::string CSimpleFunction::FunctionName(std::string functionname) const
{
  return "[CSimpleFunction::" + functionname + "]$ ";
}

